@@ -771,20 +771,22 @@ attribute(void *ctx, const xmlChar *fullname, const xmlChar *value)
     if ((!ctxt->html) && (ns == NULL) &&
         (name[0] == 'x') && (name[1] == 'm') && (name[2] == 'l') &&
         (name[3] == 'n') && (name[4] == 's') && (name[5] == 0)) {
-	xmlURIPtr uri;
+	if (value[0] != 0) {
+	    xmlURIPtr uri;
 
-	uri = xmlParseURI((const char *)value);
-	if (uri == NULL) {
-	    if ((ctxt->sax != NULL) && (ctxt->sax->warning != NULL))
-		ctxt->sax->warning(ctxt->userData, 
-		     "nmlns: %s not a valid URI\n", value);
-	} else {
-	    if (uri->scheme == NULL) {
+	    uri = xmlParseURI((const char *)value);
+	    if (uri == NULL) {
 		if ((ctxt->sax != NULL) && (ctxt->sax->warning != NULL))
 		    ctxt->sax->warning(ctxt->userData, 
-			 "nmlns: URI %s is not absolute\n", value);
+			 "nmlns: %s not a valid URI\n", value);
+	    } else {
+		if (uri->scheme == NULL) {
+		    if ((ctxt->sax != NULL) && (ctxt->sax->warning != NULL))
+			ctxt->sax->warning(ctxt->userData, 
+			     "nmlns: URI %s is not absolute\n", value);
+		}
+		xmlFreeURI(uri);
 	    }
-	    xmlFreeURI(uri);
 	}
 
 	/* a default namespace definition */
