@@ -510,7 +510,7 @@ void xmlDebugDumpOneNode(FILE *output, xmlNodePtr node, int depth) {
 	case XML_ELEMENT_NODE:
 	    fprintf(output, shift);
 	    fprintf(output, "ELEMENT ");
-	    if (node->ns != NULL) {
+	    if ((node->ns != NULL) && (node->ns->prefix != NULL)) {
 		xmlDebugDumpString(output, node->ns->prefix);
 	        fprintf(output, ":");
 	    }
@@ -1725,6 +1725,10 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 			    xmlGenericError(xmlGenericErrorContext,
 				    "%s is user-defined\n", arg);
 			    break;
+			case XPATH_XSLT_TREE:
+			    xmlGenericError(xmlGenericErrorContext,
+				    "%s is an XSLT value tree\n", arg);
+			    break;
 		    }
 		    xmlXPathFreeNodeSetList(list);
 		} else {
@@ -1785,6 +1789,10 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 			    xmlGenericError(xmlGenericErrorContext,
 				    "%s is user-defined\n", arg);
 			    break;
+			case XPATH_XSLT_TREE:
+			    xmlGenericError(xmlGenericErrorContext,
+				    "%s is an XSLT value tree\n", arg);
+			    break;
 		    }
 		    xmlXPathFreeNodeSetList(list);
 		} else {
@@ -1848,6 +1856,10 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 			    xmlGenericError(xmlGenericErrorContext,
 				    "%s is user-defined\n", arg);
 			    break;
+			case XPATH_XSLT_TREE:
+			    xmlGenericError(xmlGenericErrorContext,
+				    "%s is an XSLT value tree\n", arg);
+			    break;
 		    }
 		    xmlXPathFreeNodeSetList(list);
 		} else {
